﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using HeuristicLab.Core.Views;
using HeuristicLab.Core;
using HeuristicLab.Collections;

namespace HeuristicLab.Problems.MetaOptimization.Views {
  /// <summary>
  /// The visual representation of a <see cref="ParameterConfiguration"/>.
  /// </summary>
  [View("ParameterConfiguration View")]
  [Content(typeof(IParameterConfiguration), false)]
  [Content(typeof(ParameterConfiguration), true)]
  public partial class ParameterConfigurationView : ItemView {
    /// <summary>
    /// Gets or sets the variable to represent visually.
    /// </summary>
    /// <remarks>Uses property <see cref="ViewBase.Item"/> of base class <see cref="ViewBase"/>.
    /// No own data storage present.</remarks>
    public new IParameterConfiguration Content {
      get { return (IParameterConfiguration)base.Content; }
      set { base.Content = value; }
    }

    public ParameterConfigurationView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null) {
        optimizeCheckBox.Checked = Content.OptimizationEnabled;
        valueViewHost.ViewType = typeof(ValueView);
        valueViewHost.Content = Content;
        if (Content is IntValueParameterConfiguration) {
          configurationViewHost.Content = ((IntValueParameterConfiguration)Content).Ranges;
        } else {
          configurationViewHost.Content = null;
        }

        childParameterConfigurationListView.Content = ((ParameterConfiguration)Content).ChildParameterConfigurations;
      } else {
        valueViewHost.Content = null;
        configurationViewHost.Content = null;
        childParameterConfigurationListView.Content = null;
      }
      SetEnabledStateOfControls();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ChildParameterConfigurations.ItemsAdded += new CollectionItemsChangedEventHandler<IndexedItem<IParameterConfiguration>>(ChildParameterConfigurations_ItemsAdded);
      Content.ChildParameterConfigurations.ItemsRemoved += new CollectionItemsChangedEventHandler<IndexedItem<IParameterConfiguration>>(ChildParameterConfigurations_ItemsRemoved);
    }
    protected override void DeregisterContentEvents() {
      Content.ChildParameterConfigurations.ItemsAdded -= new CollectionItemsChangedEventHandler<IndexedItem<IParameterConfiguration>>(ChildParameterConfigurations_ItemsAdded);
      Content.ChildParameterConfigurations.ItemsRemoved -= new CollectionItemsChangedEventHandler<IndexedItem<IParameterConfiguration>>(ChildParameterConfigurations_ItemsRemoved);
      base.DeregisterContentEvents();
    }

    void ChildParameterConfigurations_ItemsRemoved(object sender, Collections.CollectionItemsChangedEventArgs<Collections.IndexedItem<IParameterConfiguration>> e) {
      SetEnabledStateOfControls();
    }
    
    void ChildParameterConfigurations_ItemsAdded(object sender, Collections.CollectionItemsChangedEventArgs<Collections.IndexedItem<IParameterConfiguration>> e) {
      SetEnabledStateOfControls();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      this.SuspendLayout();
      if (optimizeCheckBox.Checked) {
        DisableTabPage(valueTabPage);
        EnableTabPage(configurationtabPage);
        if (Content.ChildParameterConfigurations.Count > 0)
          EnableTabPage(childParameterTabPage);
      } else {
        DisableTabPage(configurationtabPage);
        DisableTabPage(childParameterTabPage);
        EnableTabPage(valueTabPage);
      }
      this.ResumeLayout();
    }

    private void optimizeCheckBox_CheckedChanged(object sender, EventArgs e) {
      Content.OptimizationEnabled = optimizeCheckBox.Checked;
      SetEnabledStateOfControls();
    }

    protected void EnableTabPage(TabPage tabPage) {
      if (!tabControl.TabPages.Contains(tabPage)) {
        tabControl.TabPages.Add(tabPage);
      }
    }

    protected void DisableTabPage(TabPage tabPage) {
      if (tabControl.TabPages.Contains(tabPage)) {
        tabControl.TabPages.Remove(tabPage);
      }
    }
  }
}
